mod boot_file_size_option;
mod cookie_server_option;
mod domain_name_server_option;
mod host_name_option;
mod impress_server_option;
mod log_server_option;
mod lpr_server_option;
mod name_server_option;
mod resource_location_server_option;
mod router_option;
mod subnet_mask;
mod time_offset;
mod time_server_option;
mod merit_dump_file;
mod domain_name;
mod swap_server;
mod root_path;
mod extensions_path;

pub use boot_file_size_option::BootFileSizeOption;
pub use cookie_server_option::CookieServerOption;
pub use domain_name_server_option::DomainNameServerOption;
pub use impress_server_option::ImpressServerOption;
pub use log_server_option::LogServerOption;
pub use lpr_server_option::LPRServerOption;
pub use name_server_option::NameServerOption;
pub use resource_location_server_option::ResourceLocationServerOption;
pub use router_option::RouterOption;
pub use subnet_mask::SubnetMask;
pub use time_offset::TimeOffset;
pub use merit_dump_file::MeritDumpFile;
pub use domain_name::DomainName;
pub use swap_server::SwapServer;
pub use root_path::RootPath;
pub use extensions_path::ExtensionsPath;

#[derive(Debug)]
pub struct RawDhcpOption<'a> {
    pub code: u8,
    pub length: u8,
    pub data: &'a [u8],
}

pub trait DhcpOption {
    const CODE: u8;
    type Item;

    fn parse(option: &RawDhcpOption) -> Result<Self::Item, DhcpOptionError>;
}

#[derive(Debug)]
pub enum DhcpOptionError {
    InvalidLength(u8, u8),
    UTF8Error,
}

impl core::fmt::Display for DhcpOptionError {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        match self {
            DhcpOptionError::InvalidLength(got, should) => {
                write!(f, "Invalid length got {got}, should be {should}")
            }
            DhcpOptionError::UTF8Error => write!(f, "Error parsing option data as UTF-8"),
        }
    }
}

impl core::error::Error for DhcpOptionError {}

impl From<core::str::Utf8Error> for DhcpOptionError {
    fn from(value: core::str::Utf8Error) -> Self {
        DhcpOptionError::UTF8Error
    }
}